using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace CircleCollisionTest
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Texture2D CircleSprite;
        Texture2D SquareSprite;
        BoundingSphere CircleCollision;
        BoundingBox SquareCollision;
        Vector2 CirclePosition;
        Vector2 SquarePosition;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            CircleSprite = Content.Load<Texture2D>("Circle");
            SquareSprite = Content.Load<Texture2D>("Square");

            CirclePosition = new Vector2(200, 200);
            SquarePosition = new Vector2(300, 100);
                       
            Vector3 aCircleCenter = new Vector3(CircleSprite.Width / 2 + CirclePosition.X, CircleSprite.Height / 2 + CirclePosition.Y, 0);
            CircleCollision = new BoundingSphere(aCircleCenter, CircleSprite.Width/2);

            Vector3 aSquareMin = new Vector3(SquarePosition.X, SquarePosition.Y, 0);
            Vector3 aSquareMax = new Vector3(SquarePosition.X + SquareSprite.Width, SquarePosition.Y + SquareSprite.Height, 0);
            SquareCollision = new BoundingBox(aSquareMin, aSquareMax);
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState CurrentKeyboard = Keyboard.GetState();
            if (CurrentKeyboard.IsKeyDown(Keys.Up))
            {
                CirclePosition.Y -= 2;
            }

            if (CurrentKeyboard.IsKeyDown(Keys.Down))
            {
                CirclePosition.Y += 2;
            }

            if (CurrentKeyboard.IsKeyDown(Keys.Left))
            {
                CirclePosition.X -= 2;
            }

            if (CurrentKeyboard.IsKeyDown(Keys.Right))
            {
                CirclePosition.X += 2;
            }

            Vector3 aCircleCenter = new Vector3(CircleSprite.Width / 2 + CirclePosition.X, CircleSprite.Height / 2 + CirclePosition.Y, 0);
            CircleCollision = new BoundingSphere(aCircleCenter, CircleSprite.Width / 2);
    

            // TODO: Add your update logic here
            if (SquareCollision.Intersects(CircleCollision))
            {
                this.Window.Title = "Collision!";
            }
            else
            {
                this.Window.Title = "They do not intersect";
            }

            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
            spriteBatch.Begin();
            spriteBatch.Draw(CircleSprite, CirclePosition, new Rectangle(0, 0, CircleSprite.Width, CircleSprite.Height),  Color.White, 0.0f, new Vector2(CircleSprite.Width/2, CircleSprite.Height/2), 1.0f, SpriteEffects.None, 0);
            spriteBatch.Draw(SquareSprite, SquarePosition, new Rectangle(0, 0, SquareSprite.Width, SquareSprite.Height), Color.White, 0.0f, new Vector2(SquareSprite.Width / 2, SquareSprite.Height / 2), 1.0f, SpriteEffects.None, 0);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
